@@ -378,26 +378,18 @@ INTERNAL int c128_hrt_cpy_iso8859_1_test(struct zint_symbol *symbol, const unsig
 
 /* Handle Code 128, 128B and HIBC 128 */
 INTERNAL int code128(struct zint_symbol *symbol, unsigned char source[], int length) {
-    int i, j, k, values[C128_MAX] = {0}, bar_characters, read, total_sum;
-    int error_number, indexchaine, indexliste, f_state;
-    int sourcelen;
+    int i, j, k, values[C128_MAX] = {0}, bar_characters = 0, read, total_sum;
+    int error_number = 0, indexchaine, indexliste, f_state = 0;
     int list[2][C128_MAX] = {{0}};
     char set[C128_MAX] = {0}, fset[C128_MAX], mode, last_set, current_set = ' ';
-    float glyph_count;
+    int glyph_count = 0; /* Codeword estimate times 2 */
     char dest[1000];
     char *d = dest;
 
     /* Suppresses clang-analyzer-core.UndefinedBinaryOperatorResult warning on fset which is fully set */
     assert(length > 0);
 
-    error_number = 0;
-
-    sourcelen = length;
-
-    bar_characters = 0;
-    f_state = 0;
-
-    if (sourcelen > C128_MAX) {
+    if (length > C128_MAX) {
         /* This only blocks ridiculously long input - the actual length of the
            resulting barcode depends on the type of data, so this is trapped later */
         sprintf(symbol->errtxt, "340: Input too long (%d character maximum)", C128_MAX);
@@ -405,13 +397,13 @@ INTERNAL int code128(struct zint_symbol *symbol, unsigned char source[], int len
     }
 
     /* Detect extended ASCII characters */
-    for (i = 0; i < sourcelen; i++) {
+    for (i = 0; i < length; i++) {
         fset[i] = source[i] >= 128 ? 'f' : ' ';
     }
 
     /* Decide when to latch to extended mode - Annex E note 3 */
     j = 0;
-    for (i = 0; i < sourcelen; i++) {
+    for (i = 0; i < length; i++) {
         if (fset[i] == 'f') {
             j++;
         } else {
@@ -424,7 +416,7 @@ INTERNAL int code128(struct zint_symbol *symbol, unsigned char source[], int len
             }
         }
 
-        if ((j >= 3) && (i == (sourcelen - 1))) {
+        if ((j >= 3) && (i == (length - 1))) {
             for (k = i; k > (i - 3); k--) {
                 fset[k] = 'F';
             }
@@ -432,14 +424,14 @@ INTERNAL int code128(struct zint_symbol *symbol, unsigned char source[], int len
     }
 
     /* Decide if it is worth reverting to 646 encodation for a few characters as described in 4.3.4.2 (d) */
-    for (i = 1; i < sourcelen; i++) {
+    for (i = 1; i < length; i++) {
         if ((fset[i - 1] == 'F') && (fset[i] == ' ')) {
             /* Detected a change from 8859-1 to 646 - count how long for */
-            for (j = 0; ((i + j) < sourcelen) && (fset[i + j] == ' '); j++);
+            for (j = 0; ((i + j) < length) && (fset[i + j] == ' '); j++);
             /* Count how many 8859-1 beyond */
             k = 0;
-            if (i + j < sourcelen) {
-                for (k = 1; ((i + j + k) < sourcelen) && (fset[i + j + k] != ' '); k++);
+            if (i + j < length) {
+                for (k = 1; ((i + j + k) < length) && (fset[i + j + k] != ' '); k++);
             }
             if (j < 3 || (j < 5 && k > 2)) {
                 /* Change to shifting back rather than latching back */
@@ -456,25 +448,25 @@ INTERNAL int code128(struct zint_symbol *symbol, unsigned char source[], int len
     indexchaine = 0;
 
     mode = c128_parunmodd(source[indexchaine]);
-    if ((symbol->symbology == BARCODE_CODE128B) && (mode == C128_ABORC)) {
+    if ((symbol->symbology == BARCODE_CODE128AB) && (mode == C128_ABORC)) {
         mode = C128_AORB;
     }
 
     do {
         list[1][indexliste] = mode;
-        while ((list[1][indexliste] == mode) && (indexchaine < sourcelen)) {
+        while ((list[1][indexliste] == mode) && (indexchaine < length)) {
             list[0][indexliste]++;
             indexchaine++;
-            if (indexchaine == sourcelen) {
+            if (indexchaine == length) {
                 break;
             }
             mode = c128_parunmodd(source[indexchaine]);
-            if ((symbol->symbology == BARCODE_CODE128B) && (mode == C128_ABORC)) {
+            if ((symbol->symbology == BARCODE_CODE128AB) && (mode == C128_ABORC)) {
                 mode = C128_AORB;
             }
         }
         indexliste++;
-    } while (indexchaine < sourcelen);
+    } while (indexchaine < length);
 
     c128_dxsmooth(list, &indexliste);
 
@@ -503,48 +495,47 @@ INTERNAL int code128(struct zint_symbol *symbol, unsigned char source[], int len
     c128_put_in_set(list, indexliste, set, NULL /*source*/);
 
     if (symbol->debug & ZINT_DEBUG_PRINT) {
-        printf("Data: %.*s (%d)\n", sourcelen, source, sourcelen);
-        printf(" Set: %.*s\n", sourcelen, set);
-        printf("FSet: %.*s\n", sourcelen, fset);
+        printf("Data: %.*s (%d)\n", length, source, length);
+        printf(" Set: %.*s\n", length, set);
+        printf("FSet: %.*s\n", length, fset);
     }
 
     /* Now we can calculate how long the barcode is going to be - and stop it from
        being too long */
     last_set = set[0];
-    glyph_count = 0.0f;
-    for (i = 0; i < sourcelen; i++) {
+    for (i = 0; i < length; i++) {
         if ((set[i] == 'a') || (set[i] == 'b')) {
-            glyph_count = glyph_count + 1.0f;
+            glyph_count += 2; /* 1 codeword */
         }
         if ((fset[i] == 'f') || (fset[i] == 'n')) {
-            glyph_count = glyph_count + 1.0f;
+            glyph_count += 2; /* May be overestimate if in latch */
         }
         if (((set[i] == 'A') || (set[i] == 'B')) || (set[i] == 'C')) {
             if (set[i] != last_set) {
                 last_set = set[i];
-                glyph_count = glyph_count + 1.0f;
+                glyph_count += 2;
             }
         }
         if (i == 0) {
             if (fset[i] == 'F') {
-                glyph_count = glyph_count + 2.0f;
+                glyph_count += 4; /* 2 codewords */
             }
         } else {
             if ((fset[i] == 'F') && (fset[i - 1] != 'F')) {
-                glyph_count = glyph_count + 2.0f;
+                glyph_count += 4;
             }
             if ((fset[i] != 'F') && (fset[i - 1] == 'F')) {
-                glyph_count = glyph_count + 2.0f;
+                glyph_count += 4;
             }
         }
 
         if (set[i] == 'C') {
-            glyph_count = glyph_count + 0.5f;
+            glyph_count += 1; /* Half a codeword */
         } else {
-            glyph_count = glyph_count + 1.0f;
+            glyph_count += 2;
         }
     }
-    if (glyph_count > 60.0f) {
+    if (glyph_count > 120) { /* 60 * 2 */
         strcpy(symbol->errtxt, "341: Input too long (60 symbol character maximum)");
         return ZINT_ERROR_TOO_LONG;
     }
@@ -593,13 +584,14 @@ INTERNAL int code128(struct zint_symbol *symbol, unsigned char source[], int len
             case 'A':
                 values[bar_characters++] = 101;
                 values[bar_characters++] = 101;
+                f_state = 1;
                 break;
             case 'B':
                 values[bar_characters++] = 100;
                 values[bar_characters++] = 100;
+                f_state = 1;
                 break;
         }
-        f_state = 1;
     }
 
     /* Encode the data */
@@ -631,13 +623,14 @@ INTERNAL int code128(struct zint_symbol *symbol, unsigned char source[], int len
                     case 'A':
                         values[bar_characters++] = 101;
                         values[bar_characters++] = 101;
+                        f_state = 1;
                         break;
                     case 'B':
                         values[bar_characters++] = 100;
                         values[bar_characters++] = 100;
+                        f_state = 1;
                         break;
                 }
-                f_state = 1;
             }
             if ((fset[read] == ' ') && (f_state == 1)) {
                 /* Latch end of extended mode */
@@ -645,17 +638,18 @@ INTERNAL int code128(struct zint_symbol *symbol, unsigned char source[], int len
                     case 'A':
                         values[bar_characters++] = 101;
                         values[bar_characters++] = 101;
+                        f_state = 0;
                         break;
                     case 'B':
                         values[bar_characters++] = 100;
                         values[bar_characters++] = 100;
+                        f_state = 0;
                         break;
                 }
-                f_state = 0;
             }
         }
 
-        if ((fset[read] == 'f') || (fset[read] == 'n')) {
+        if ((fset[read] == 'f' && f_state == 0) || (fset[read] == 'n' && f_state == 1)) {
             /* Shift to or from extended mode */
             switch (current_set) {
                 case 'A':
@@ -686,7 +680,7 @@ INTERNAL int code128(struct zint_symbol *symbol, unsigned char source[], int len
                 break;
         }
 
-    } while (read < sourcelen);
+    } while (read < length);
 
     /* Destination setting and check digit calculation */
     memcpy(d, C128Table[values[0]], 6);
@@ -714,6 +708,7 @@ INTERNAL int code128(struct zint_symbol *symbol, unsigned char source[], int len
         }
         printf(" (%d)\n", bar_characters);
         printf("Barspaces: %.*s\n", (int) (d - dest), dest);
+        printf("Checksum:  %d\n", total_sum);
     }
 #ifdef ZINT_TEST
     if (symbol->debug & ZINT_DEBUG_TEST) {
@@ -733,22 +728,17 @@ INTERNAL int code128(struct zint_symbol *symbol, unsigned char source[], int len
 /* Handle EAN-128 (Now known as GS1-128), and composite version if `cc_mode` set */
 INTERNAL int gs1_128_cc(struct zint_symbol *symbol, unsigned char source[], int length, const int cc_mode,
                 const int cc_rows) {
-    int i, values[C128_MAX] = {0}, bar_characters, read, total_sum;
+    int i, values[C128_MAX] = {0}, bar_characters = 0, read, total_sum;
     int error_number, warn_number = 0, indexchaine, indexliste;
     int list[2][C128_MAX] = {{0}};
     char set[C128_MAX] = {0}, mode, last_set;
-    float glyph_count;
+    int glyph_count = 0; /* Codeword estimate times 2 */
     char dest[1000];
     char *d = dest;
-    int separator_row, linkage_flag;
+    int separator_row = 0, linkage_flag = 0;
     int reduced_length;
     unsigned char *reduced = (unsigned char *) z_alloca(length + 1);
 
-    linkage_flag = 0;
-
-    bar_characters = 0;
-    separator_row = 0;
-
     if (length > C128_MAX) {
         /* This only blocks ridiculously long input - the actual length of the
         resulting barcode depends on the type of data, so this is trapped later */
@@ -805,24 +795,23 @@ INTERNAL int gs1_128_cc(struct zint_symbol *symbol, unsigned char source[], int
     /* Now we can calculate how long the barcode is going to be - and stop it from
        being too long */
     last_set = set[0];
-    glyph_count = 0.0f;
     for (i = 0; i < reduced_length; i++) {
         if ((set[i] == 'A') || (set[i] == 'B') || (set[i] == 'C')) {
             if (set[i] != last_set) {
                 last_set = set[i];
-                glyph_count = glyph_count + 1.0f;
+                glyph_count += 2; /* 1 codeword */
             }
         } else if ((set[i] == 'a') || (set[i] == 'b')) {
-            glyph_count = glyph_count + 1.0f; /* Not reached */
+            glyph_count += 2; /* Not reached */
         }
 
         if ((set[i] == 'C') && (reduced[i] != '[')) {
-            glyph_count = glyph_count + 0.5f;
+            glyph_count += 1; /* Half a codeword */
         } else {
-            glyph_count = glyph_count + 1.0f;
+            glyph_count += 2;
         }
     }
-    if (glyph_count > 60.0f) {
+    if (glyph_count > 120) { /* 60 * 2 */
         strcpy(symbol->errtxt, "344: Input too long (60 symbol character maximum)");
         return ZINT_ERROR_TOO_LONG;
     }
@@ -1063,57 +1052,89 @@ INTERNAL int ean14(struct zint_symbol *symbol, unsigned char source[], int lengt
 }
 
 /* DPD (Deutscher Paketdienst) Code */
-/* Specification at ftp://dpd.at/Datenspezifikationen/EN/gbs_V4.0.2_hauptdokument.pdf
- * or https://docplayer.net/33728877-Dpd-parcel-label-specification.html */
+/* Specification at https://esolutions.dpd.com/dokumente/DPD_Parcel_Label_Specification_2.4.1_EN.pdf
+ * and identification tag info (Barcode ID) at https://esolutions.dpd.com/dokumente/DPD_Routing_Database_1.3_EN.pdf */
 INTERNAL int dpd(struct zint_symbol *symbol, unsigned char source[], int length) {
     int error_number = 0;
     int i, p;
-    unsigned char identifier;
+    unsigned char ident_tag;
+    unsigned char local_source_buf[29];
+    unsigned char *local_source;
     const int mod = 36;
+    const int relabel = symbol->option_2 == 1; /* A "relabel" has no identification tag */
     int cd; /* Check digit */
 
-    if (length != 28) {
-        strcpy(symbol->errtxt, "349: DPD input wrong length (28 characters required)");
+    if ((length != 27 && length != 28) || (length == 28 && relabel)) {
+        if (relabel) {
+            strcpy(symbol->errtxt, "830: DPD relabel input wrong length (27 characters required)");
+        } else {
+            strcpy(symbol->errtxt, "349: DPD input wrong length (27 or 28 characters required)");
+        }
         return ZINT_ERROR_TOO_LONG;
     }
 
-    identifier = source[0];
+    if (length == 27 && !relabel) {
+        local_source_buf[0] = '%';
+        ustrcpy(local_source_buf + 1, source);
+        local_source = local_source_buf;
+        length++;
+    } else {
+        local_source = source;
+    }
 
-    to_upper(source + 1, length - 1);
-    if (!is_sane(KRSET_F, source + 1, length - 1)) {
-        strcpy(symbol->errtxt, "300: Invalid character in DPD data (alphanumerics only)");
+    ident_tag = local_source[0];
+
+    to_upper(local_source + !relabel, length - !relabel);
+    if (!is_sane(KRSET_F, local_source + !relabel, length - !relabel)) {
+        if (local_source == local_source_buf || relabel) {
+            strcpy(symbol->errtxt, "300: Invalid character in data (alphanumerics only)");
+        } else {
+            strcpy(symbol->errtxt, "299: Invalid character in data (alphanumerics only after first character)");
+        }
         return ZINT_ERROR_INVALID_DATA;
     }
 
-    if ((identifier < 32) || (identifier > 127)) {
-        strcpy(symbol->errtxt, "343: Invalid DPD identifier (first character), ASCII values 32 to 127 only");
+    if ((ident_tag < 32) || (ident_tag > 127)) {
+        strcpy(symbol->errtxt, "343: Invalid DPD identification tag (first character), ASCII values 32 to 127 only");
         return ZINT_ERROR_INVALID_DATA;
     }
 
-    error_number = code128(symbol, source, length); /* Only returns errors, not warnings */
+    error_number = code128(symbol, local_source, length); /* Only returns errors, not warnings */
 
     if (error_number < ZINT_ERROR) {
+        if (!(symbol->output_options & (BARCODE_BOX | BARCODE_BIND | BARCODE_BIND_TOP))) {
+            /* If no option has been selected then uses default bind top option */
+            symbol->output_options |= BARCODE_BIND_TOP; /* Note won't extend over quiet zones for DPD */
+            if (symbol->border_width == 0) { /* Allow override if non-zero */
+                symbol->border_width = 3; /* From examples, not mentioned in spec */
+            }
+        }
+
         if (symbol->output_options & COMPLIANT_HEIGHT) {
-            /* Specification DPD and primetime Parcel Despatch 4.0.2 Section 5.5.1
+            /* DPD Parcel Label Specification Version 2.4.1 (19.01.2021) Section 4.6.1.2
                25mm / 0.4mm (X max) = 62.5 min, 25mm / 0.375 (X) ~ 66.66 default */
-            error_number = set_height(symbol, 62.5f, stripf(25.0f / 0.375f), 0.0f, 0 /*no_errtxt*/);
+            if (relabel) { /* If relabel then half-size */
+                error_number = set_height(symbol, 31.25f, stripf(12.5f / 0.375f), 0.0f, 0 /*no_errtxt*/);
+            } else {
+                error_number = set_height(symbol, 62.5f, stripf(25.0f / 0.375f), 0.0f, 0 /*no_errtxt*/);
+            }
         } else {
-            (void) set_height(symbol, 0.0f, 50.0f, 0.0f, 1 /*no_errtxt*/);
+            (void) set_height(symbol, 0.0f, relabel ? 25.0f : 50.0f, 0.0f, 1 /*no_errtxt*/);
         }
 
         cd = mod;
 
         p = 0;
-        for (i = 1; i < length; i++) {
-            symbol->text[p] = source[i];
+        for (i = !relabel; i < length; i++) {
+            symbol->text[p] = local_source[i];
             p++;
 
-            cd += posn(KRSET, source[i]);
+            cd += posn(KRSET, local_source[i]);
             if (cd > mod) cd -= mod;
             cd *= 2;
             if (cd >= (mod + 1)) cd -= mod + 1;
 
-            switch (i) {
+            switch (i + relabel) {
                 case 4:
                 case 7:
                 case 11:
@@ -1139,9 +1160,121 @@ INTERNAL int dpd(struct zint_symbol *symbol, unsigned char source[], int length)
         p++;
 
         symbol->text[p] = '\0';
+
+        if (error_number == 0) {
+            /* Some compliance checks */
+            if (!is_sane(NEON_F, local_source + length - 16, 16)) {
+                if (!is_sane(NEON_F, local_source + length - 3, 3)) { /* 3-digit Country Code (ISO 3166-1) */
+                    strcpy(symbol->errtxt, "831: Destination Country Code (last 3 characters) should be numeric");
+                } else if (!is_sane(NEON_F, local_source + length - 6, 3)) { /* 3-digit Service Code */
+                    strcpy(symbol->errtxt, "832: Service Code (characters 6-4 from end) should be numeric");
+                } else { /* Last 10 characters of Tracking No. */
+                    strcpy(symbol->errtxt,
+                        "833: Last 10 characters of Tracking Number (characters 16-7 from end) should be numeric");
+                }
+                error_number = ZINT_WARN_NONCOMPLIANT;
+            }
+        }
     }
 
     return error_number;
 }
 
+/* Universal Postal Union S10 */
+/* https://www.upu.int/UPU/media/upu/files/postalSolutions/programmesAndServices/standards/S10-12.pdf */
+INTERNAL int upu_s10(struct zint_symbol *symbol, unsigned char source[], int length) {
+    int i, j;
+    unsigned char local_source[13 + 1];
+    unsigned char have_check_digit = '\0';
+    int check_digit;
+    static char weights[8] = { 8, 6, 4, 2, 3, 5, 9, 7 };
+    int error_number = 0, warn_number = 0;
+
+    if (length != 12 && length != 13) {
+        strcpy(symbol->errtxt, "834: Input must be 12 or 13 characters long");
+        return ZINT_ERROR_TOO_LONG;
+    }
+    if (length == 13) { /* Includes check digit - remove for now */
+        have_check_digit = source[10];
+        memcpy(local_source, source, 10);
+        ustrcpy(local_source + 10, source + 11);
+    } else {
+        ustrcpy(local_source, source);
+    }
+    to_upper(local_source, length);
+
+    if (!z_isupper(local_source[0]) || !z_isupper(local_source[1])) {
+        strcpy(symbol->errtxt, "835: Invalid character in Service Indictor (first 2 characters) (alphabetic only)");
+        return ZINT_ERROR_INVALID_DATA;
+    }
+    if (!is_sane(NEON_F, local_source + 2, 12 - 4) || (have_check_digit && !z_isdigit(have_check_digit))) {
+        sprintf(symbol->errtxt, "836: Invalid character in Serial Number (middle %d characters) (digits only)",
+                have_check_digit ? 9 : 8);
+        return ZINT_ERROR_INVALID_DATA;
+    }
+    if (!z_isupper(local_source[10]) || !z_isupper(local_source[11])) {
+        strcpy(symbol->errtxt, "837: Invalid character in Country Code (last 2 characters) (alphabetic only)");
+        return ZINT_ERROR_INVALID_DATA;
+    }
+
+    check_digit = 0;
+    for (i = 2; i < 10; i++) { /* Serial Number only */
+        check_digit += ctoi(local_source[i]) * weights[i - 2];
+    }
+    check_digit %= 11;
+    check_digit = 11 - check_digit;
+    if (check_digit == 10) {
+        check_digit = 0;
+    } else if (check_digit == 11) {
+        check_digit = 5;
+    }
+    if (have_check_digit && ctoi(have_check_digit) != check_digit) {
+        sprintf(symbol->errtxt, "838: Invalid check digit '%c', expecting '%c'", have_check_digit, itoc(check_digit));
+        return ZINT_ERROR_INVALID_CHECK;
+    }
+    /* Add in (back) check digit */
+    local_source[12] = local_source[11];
+    local_source[11] = local_source[10];
+    local_source[10] = itoc(check_digit);
+    local_source[13] = '\0';
+
+    /* Do some checks on the Service Indicator (first char only) and Country Code */
+    if (strchr("JKSTW", local_source[0]) != NULL) { /* These are reserved & cannot be assigned */
+        strcpy(symbol->errtxt, "839: Invalid Service Indicator (first character should not be any of \"JKSTW\")");
+        error_number = ZINT_WARN_NONCOMPLIANT;
+    } else if (strchr("FHIOXY", local_source[0]) != NULL) { /* These aren't allocated as of spec Oct 2017 */
+        strcpy(symbol->errtxt, "840: Non-standard Service Indicator (first 2 characters)");
+        error_number = ZINT_WARN_NONCOMPLIANT;
+    } else if (!gs1_iso3166_alpha2(local_source + 11)) {
+        strcpy(symbol->errtxt, "841: Country code (last two characters) is not ISO 3166-1");
+        error_number = ZINT_WARN_NONCOMPLIANT;
+    }
+
+    (void) code128(symbol, local_source, 13); /* Only error returned is TOO_LONG which can't happen */
+
+    j = 0;
+    for (i = 0; i < 13; i++) {
+        if (i == 2 || i == 5 || i == 8 || i == 11) {
+            symbol->text[j++] = ' ';
+        }
+        symbol->text[j++] = local_source[i];
+    }
+    symbol->text[j] = '\0';
+
+    if (symbol->output_options & COMPLIANT_HEIGHT) {
+        /* Universal Postal Union S10 Section 8, using max X 0.51mm & minimum height 12.5mm or 15% of width */
+        const float min_height_min = stripf(12.5f / 0.51f);
+        float min_height = stripf(symbol->width * 0.15f);
+        if (min_height < min_height_min) {
+            min_height = min_height_min;
+        }
+        /* Using 50 as default as none recommended */
+        warn_number = set_height(symbol, min_height, min_height > 50.0f ? min_height : 50.0f, 0.0f, 0 /*no_errtxt*/);
+    } else {
+        (void) set_height(symbol, 0.0f, 50.0f, 0.0f, 1 /*no_errtxt*/);
+    }
+
+    return error_number ? error_number : warn_number;
+}
+
 /* vim: set ts=4 sw=4 et : */
