@@ -1,6 +1,6 @@
 /*
     libzint - the open source barcode library
-    Copyright (C) 2020 - 2021 Robin Stuart <rstuart114@gmail.com>
+    Copyright (C) 2020-2022 Robin Stuart <rstuart114@gmail.com>
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -27,10 +27,10 @@
     OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
     SUCH DAMAGE.
  */
-/* vim: set ts=4 sw=4 et : */
+/* SPDX-License-Identifier: BSD-3-Clause */
 
-#ifndef CODE128_H
-#define CODE128_H
+#ifndef Z_CODE128_H
+#define Z_CODE128_H
 
 #ifdef __cplusplus
 extern "C" {
@@ -63,4 +63,5 @@ INTERNAL_DATA_EXTERN const char C128Table[107][6];
 }
 #endif /* __cplusplus */
 
-#endif /* CODE128_H */
+/* vim: set ts=4 sw=4 et : */
+#endif /* Z_CODE128_H */
