@@ -758,7 +758,9 @@ static int dm_getEndMode(struct zint_symbol *symbol, const unsigned char *source
     return mode;
 }
 
-/*#define DM_TRACE*/
+#if 0
+#define DM_TRACE
+#endif
 #include "dmatrix_trace.h"
 
 /* Return number of C40/TEXT codewords needed to encode characters in full batches of 3 (or less if EOD).
@@ -1029,6 +1031,7 @@ static int dm_define_mode(struct zint_symbol *symbol, char modes[], const unsign
             mode_len = 0;
         }
     }
+
     if (debug_print) {
         printf("modes (%d): ", length);
         for (i = 0; i < length; i++) printf("%c", dm_smodes[(int) modes[i]][0]);
@@ -1237,6 +1240,7 @@ static int dm_minimalenc(struct zint_symbol *symbol, const unsigned char source[
 static int dm_isoenc(struct zint_symbol *symbol, const unsigned char source[], const int length, int *p_sp,
             unsigned char target[], int *p_tp, int process_buffer[8], int *p_process_p, int *p_b256_start,
             int *p_current_mode, const int gs1, const int debug_print) {
+    const int mailmark = symbol->symbology == BARCODE_MAILMARK_2D;
     int sp = *p_sp;
     int tp = *p_tp;
     int process_p = *p_process_p;
@@ -1247,6 +1251,23 @@ static int dm_isoenc(struct zint_symbol *symbol, const unsigned char source[], c
     /* step (a) */
     int next_mode = DM_ASCII;
 
+    if (mailmark) { /* First 45 characters C40 */
+        assert(length >= 45);
+        next_mode = DM_C40;
+        tp = dm_switch_mode(next_mode, target, tp, p_b256_start, debug_print);
+        while (sp < 45) {
+            assert(!(sp & 0x80));
+            process_buffer[process_p++] = dm_c40_value[source[sp]];
+
+            if (process_p >= 3) {
+                process_p = dm_ctx_buffer_xfer(process_buffer, process_p, target, &tp, debug_print);
+            }
+            sp++;
+        }
+        current_mode = next_mode;
+        not_first = 1;
+    }
+
     while (sp < length) {
 
         current_mode = next_mode;
@@ -1357,6 +1378,7 @@ static int dm_isoenc(struct zint_symbol *symbol, const unsigned char source[], c
             }
 
             if (next_mode != DM_X12) {
+                sp -= process_p; /* About to throw away buffer, need to re-process input, cf Okapi commit [fb7981e] */
                 process_p = 0; /* Throw away buffer if any */
                 target[tp++] = 254; /* Unlatch */
                 next_mode = DM_ASCII;
@@ -1390,7 +1412,7 @@ static int dm_isoenc(struct zint_symbol *symbol, const unsigned char source[], c
             } else {
                 next_mode = DM_EDIFACT;
                 if (process_p == 3) {
-                    /* Note different then spec Step (f)(2), which suggests checking when 0, but this seems to
+                    /* Note different than spec Step (f)(2), which suggests checking when 0, but this seems to
                        work better in many cases as the switch to ASCII is "free" */
                     next_mode = dm_look_ahead_test(source, length, sp, current_mode, process_p, gs1, debug_print);
                 }
@@ -1499,7 +1521,8 @@ static int dm_encode(struct zint_symbol *symbol, const unsigned char source[], c
         if (debug_print) printf("ECI %d ", eci + 1);
     }
 
-    if (symbol->input_mode & FAST_MODE) { /* If FAST_MODE, do Annex J-based encodation */
+    /* If FAST_MODE or MAILMARK_2D, do Annex J-based encodation */
+    if ((symbol->input_mode & FAST_MODE) || symbol->symbology == BARCODE_MAILMARK_2D) {
         error_number = dm_isoenc(symbol, source, length, &sp, target, &tp, process_buffer, &process_p,
                                     &b256_start, &current_mode, gs1, debug_print);
     } else { /* Do default minimal encodation */
@@ -1665,7 +1688,7 @@ static int dm_encode_segs(struct zint_symbol *symbol, struct zint_seg segs[], co
     const int debug_print = symbol->debug & ZINT_DEBUG_PRINT;
 
     if (segs_length(segs, seg_count) > 3116) { /* Max is 3166 digits */
-        strcpy(symbol->errtxt, "760: Data too long to fit in symbol");
+        strcpy(symbol->errtxt, "719: Data too long to fit in symbol");
         return ZINT_ERROR_TOO_LONG;
     }
 
@@ -1683,7 +1706,7 @@ static int dm_encode_segs(struct zint_symbol *symbol, struct zint_seg segs[], co
         if (symbol->structapp.id[0]) {
             int id, id_len, id1_err, id2_err;
 
-            for (id_len = 0; id_len < 32 && symbol->structapp.id[id_len]; id_len++);
+            for (id_len = 1; id_len < 7 && symbol->structapp.id[id_len]; id_len++);
 
             if (id_len > 6) { /* ID1 * 1000 + ID2 */
                 strcpy(symbol->errtxt, "722: Structured Append ID too long (6 digit maximum)");
