@@ -1,7 +1,7 @@
-/*  output.h - Common routines for raster/vector
-
+/*  output.h - Common routines for raster/vector */
+/*
     libzint - the open source barcode library
-    Copyright (C) 2020 - 2021 Robin Stuart <rstuart114@gmail.com>
+    Copyright (C) 2020-2022 Robin Stuart <rstuart114@gmail.com>
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -28,7 +28,7 @@
     OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
     SUCH DAMAGE.
  */
-/* vim: set ts=4 sw=4 et : */
+/* SPDX-License-Identifier: BSD-3-Clause */
 
 #ifndef Z_OUTPUT_H
 #define Z_OUTPUT_H
@@ -37,6 +37,8 @@
 extern "C" {
 #endif /* __cplusplus */
 
+#include <stdio.h> /* For FILE */
+
 INTERNAL int out_check_colour_options(struct zint_symbol *symbol);
 INTERNAL void out_set_whitespace_offsets(const struct zint_symbol *symbol, const int hide_text,
                 float *xoffset, float *yoffset, float *roffset, float *boffset, const float scaler,
@@ -47,9 +49,11 @@ INTERNAL float out_large_bar_height(struct zint_symbol *symbol, int si, int *row
 INTERNAL void out_upcean_split_text(int upceanflag, unsigned char text[],
                 unsigned char textpart1[5], unsigned char textpart2[7], unsigned char textpart3[7],
                 unsigned char textpart4[2]);
+INTERNAL FILE *out_fopen(const char filename[256], const char *mode);
 
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
 
+/* vim: set ts=4 sw=4 et : */
 #endif /* Z_OUTPUT_H */
