@@ -1,8 +1,7 @@
 /* telepen.c - Handles Telepen and Telepen numeric */
-
 /*
     libzint - the open source barcode library
-    Copyright (C) 2008-2021 Robin Stuart <rstuart114@gmail.com>
+    Copyright (C) 2008-2022 Robin Stuart <rstuart114@gmail.com>
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -29,12 +28,12 @@
     OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
     SUCH DAMAGE.
  */
-/* vim: set ts=4 sw=4 et : */
+/* SPDX-License-Identifier: BSD-3-Clause */
 
 /* Telepen Barcode Symbology information and History (BSiH)
    https://telepen.co.uk/wp-content/uploads/2018/10/Barcode-Symbology-information-and-History.pdf */
 
-#define SODIUM_X_F        (IS_NUM_F | IS_UX__F) /* SODIUM "0123456789X" */
+#define SODIUM_X_F        (IS_NUM_F | IS_UX__F | IS_LX__F) /* SODIUM "0123456789Xx" */
 
 #include <stdio.h>
 #include "common.h"
@@ -74,6 +73,7 @@ static const char TeleTable[128][16] = {
     { "31111111111111" }, { "111311111113"   }, { "131111111113"   },
     {'1','1','1','1','1','1','1','1','1','1','1','1','1','1','1','1'},
 };
+
 /* Generated by "backend/tests/test_telepen -f generate_lens -g" */
 static const char TeleLens[128] = {
      8, 10,  8, 10, 10,  8,  8, 12,  8, 10,  8, 10, 10, 10, 10, 12,
@@ -155,7 +155,7 @@ INTERNAL int telepen_num(struct zint_symbol *symbol, unsigned char source[], int
     int i;
     char dest[521]; /* 12 (start) + 30 * 16 (max for DELs) + 16 (check digit) + 12 (stop) + 1 = 521 */
     char *d = dest;
-    unsigned char temp[64];
+    unsigned char temp[61];
 
     count = 0;
 
@@ -163,20 +163,20 @@ INTERNAL int telepen_num(struct zint_symbol *symbol, unsigned char source[], int
         strcpy(symbol->errtxt, "392: Input too long (60 character maximum)");
         return ZINT_ERROR_TOO_LONG;
     }
-    ustrcpy(temp, source);
-    to_upper(temp, src_len);
-    if (!is_sane(SODIUM_X_F, temp, src_len)) {
+    if (!is_sane(SODIUM_X_F, source, src_len)) {
         strcpy(symbol->errtxt, "393: Invalid character in data (digits and \"X\" only)");
         return ZINT_ERROR_INVALID_DATA;
     }
 
     /* Add a leading zero if required */
     if (src_len & 1) {
-        memmove(temp + 1, temp, src_len);
+        memcpy(temp + 1, source, src_len++);
         temp[0] = '0';
-
-        temp[++src_len] = '\0';
+    } else {
+        memcpy(temp, source, src_len);
     }
+    temp[src_len] = '\0';
+    to_upper(temp, src_len);
 
     /* Start character */
     memcpy(d, TeleTable['_'], 12);
@@ -224,3 +224,5 @@ INTERNAL int telepen_num(struct zint_symbol *symbol, unsigned char source[], int
     ustrcpy(symbol->text, temp);
     return error_number;
 }
+
+/* vim: set ts=4 sw=4 et : */
