@@ -37,6 +37,10 @@
 #ifndef Z_TESTCOMMON_H
 #define Z_TESTCOMMON_H
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 #define ZINT_DEBUG_TEST_PRINT           16
 #define ZINT_DEBUG_TEST_LESS_NOISY      32
 #define ZINT_DEBUG_TEST_KEEP_OUTFILE    64
@@ -53,6 +57,10 @@
 #define testutil_pclose(stream) _pclose(stream)
 #else
 #include <unistd.h>
+#  if defined(ZINT_IS_C89)
+    extern FILE *popen(const char *command, const char *type);
+    extern int pclose(FILE *stream);
+#  endif
 #define testutil_popen(command, mode) popen(command, mode)
 #define testutil_pclose(stream) pclose(stream)
 #endif
@@ -64,15 +72,11 @@
 #  pragma warning(disable: 4305) /* truncation from 'double' to 'float' */
 #endif
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
 extern int assertionFailed;
 extern int assertionNum;
 extern const char *assertionFilename;
 
-#if _MSC_VER < 1900 /* MSVC 2015 */
+#if defined(_MSC_VER) && _MSC_VER < 1900 /* MSVC 2015 */
 #define testStart(__arg__) (testStartReal("", __arg__))
 #else
 #define testStart(__arg__) (testStartReal(__func__, __arg__))
@@ -82,19 +86,30 @@ void testFinish(void);
 void testSkip(const char *msg);
 void testReport(void);
 
+#define ZINT_TEST_CTX_EXC_MAX   32
+typedef struct s_testCtx {
+    int index;
+    int index_end;
+    int exclude[ZINT_TEST_CTX_EXC_MAX];
+    int exclude_end[ZINT_TEST_CTX_EXC_MAX];
+    int generate;
+    int debug;
+} testCtx;
+typedef void (*testFunc_t)(const testCtx *const p_ctx);
 typedef struct s_testFunction {
-    const char *name; void *func; int has_index; int has_generate; int has_debug;
+    const char *name; testFunc_t func;
 } testFunction;
 void testRun(int argc, char *argv[], testFunction funcs[], int funcs_size);
+int testContinue(const testCtx *const p_ctx, const int i);
 
-#if _MSC_VER == 1200 /* VC6 */
+#if (defined(_MSC_VER) &&_MSC_VER == 1200) || defined(ZINT_IS_C89) /* VC6 or C89 */
 void assert_zero(int exp, const char *fmt, ...);
 void assert_nonzero(int exp, const char *fmt, ...);
 void assert_null(const void *exp, const char *fmt, ...);
 void assert_nonnull(const void *exp, const char *fmt, ...);
 void assert_equal(int e1, int e2, const char *fmt, ...);
 void assert_equalu64(uint64_t e1, uint64_t e2, const char *fmt, ...);
-void assert_notequal(int e1, int e2, ...);
+void assert_notequal(int e1, int e2, const char *fmt, ...);
 #else
 #define assert_exp(__exp__, ...) \
     { assertionNum++; if (!(__exp__)) { assertionFailed++; printf("%s:%d ", assertionFilename, __LINE__); \
@@ -149,6 +164,8 @@ int testUtilDirExists(const char *dirname);
 int testUtilMkDir(const char *dirname);
 int testUtilRmDir(const char *dirname);
 int testUtilRename(const char *oldpath, const char *newpath);
+int testUtilCreateROFile(const char *filename);
+int testUtilRmROFile(const char *filename);
 
 int testUtilCmpPngs(const char *file1, const char *file2);
 int testUtilCmpTxts(const char *txt1, const char *txt2);
@@ -156,8 +173,8 @@ int testUtilCmpBins(const char *bin1, const char *bin2);
 int testUtilCmpSvgs(const char *svg1, const char *svg2);
 int testUtilCmpEpss(const char *eps1, const char *eps2);
 
-int testUtilHaveIdentify(void);
-int testUtilVerifyIdentify(const char *filename, int debug);
+const char *testUtilHaveIdentify(void);
+int testUtilVerifyIdentify(const char *const prog, const char *filename, int debug);
 int testUtilHaveLibreOffice(void);
 int testUtilVerifyLibreOffice(const char *filename, int debug);
 int testUtilHaveGhostscript(void);
