@@ -14,17 +14,22 @@ function run_bwipp_test() {
 }
 
 run_bwipp_test "test_2of5" "encode"
+run_bwipp_test "test_auspost" "input"
 run_bwipp_test "test_auspost" "encode"
 run_bwipp_test "test_aztec" "encode"
 run_bwipp_test "test_aztec" "encode_segs"
 run_bwipp_test "test_bc412"
 run_bwipp_test "test_channel" "encode"
+run_bwipp_test "test_codablock" "input"
 run_bwipp_test "test_codablock" "encode"
 run_bwipp_test "test_code" "encode"
 run_bwipp_test "test_code1" "encode"
 run_bwipp_test "test_code1" "encode_segs"
+run_bwipp_test "test_code128" "input"
 run_bwipp_test "test_code128" "encode"
+run_bwipp_test "test_code16k" "input"
 run_bwipp_test "test_code16k" "encode"
+run_bwipp_test "test_code49" "input"
 run_bwipp_test "test_code49" "encode"
 run_bwipp_test "test_composite"
 run_bwipp_test "test_dmatrix" "input"
@@ -34,15 +39,19 @@ run_bwipp_test "test_dotcode" "input"
 run_bwipp_test "test_dotcode" "encode"
 run_bwipp_test "test_dotcode" "encode_segs"
 run_bwipp_test "test_gs1" "gs1_reduce"
+run_bwipp_test "test_imail" "input"
 run_bwipp_test "test_imail" "encode"
+run_bwipp_test "test_mailmark" "2d_encode"
 run_bwipp_test "test_maxicode" "input"
 run_bwipp_test "test_maxicode" "encode"
 run_bwipp_test "test_maxicode" "encode_segs"
+run_bwipp_test "test_medical" "input"
 run_bwipp_test "test_medical" "encode"
 run_bwipp_test "test_pdf417" "input"
 run_bwipp_test "test_pdf417" "encode"
 run_bwipp_test "test_pdf417" "encode_segs"
 run_bwipp_test "test_plessey" "encode"
+run_bwipp_test "test_postal" "input"
 run_bwipp_test "test_postal" "encode"
 run_bwipp_test "test_qr"
 run_bwipp_test "test_rss"
