@@ -28,6 +28,7 @@
     OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
     SUCH DAMAGE.
 */
+/* SPDX-License-Identifier: BSD-3-Clause */
 /*
  History
 
@@ -151,6 +152,18 @@
 - Added BC412
 2022-08-20 GL
 - Added CEPNet
+2022-11-10 GL
+- Added -bindtop option
+2022-12-02 GL
+- Added -scalexdimdp option
+- Renamed CODE128B to CODE128AB
+    *** Potential incompatibility ***
+2022-12-08 GL
+- Added MAILMARK_2D
+- Renamed MAILMARK to MAILMARK_4S
+    *** Potential incompatibility ***
+2022-12-09 GL
+- Added UPU_S10
 */
 
 #if defined(__WIN32__) || defined(_WIN32) || defined(WIN32)
@@ -170,7 +183,7 @@
 
 #include <zint.h>
 /* Load version defines */
-#include <zintconfig.h>
+#include "../backend/zintconfig.h"
 #include <string.h>
 
 #if defined(__WIN32__) || defined(_WIN32) || defined(WIN32)
@@ -258,7 +271,7 @@ static const char *s_code_list[] = {
     "PDF417Compact",
     "MaxiCode",
     "QR",
-    "Code128B",
+    "Code128AB",
     "AusPost",
     "AusReply",
     "AusRoute",
@@ -296,7 +309,9 @@ static const char *s_code_list[] = {
     "HIBCAztec",
     "DotCode",
     "HanXin",
-    "MailMark",
+    "MailMark-2D",
+    "UPU-S10",
+    "MailMark-4S",
     "AztecRunes",
     "Code32",
     "EAN-CC",
@@ -358,7 +373,7 @@ static const int s_code_number[] = {
     BARCODE_PDF417COMP,
     BARCODE_MAXICODE,
     BARCODE_QRCODE,
-    BARCODE_CODE128B,
+    BARCODE_CODE128AB,
     BARCODE_AUSPOST,
     BARCODE_AUSREPLY,
     BARCODE_AUSROUTE,
@@ -396,7 +411,9 @@ static const int s_code_number[] = {
     BARCODE_HIBC_AZTEC,
     BARCODE_DOTCODE,
     BARCODE_HANXIN,
-    BARCODE_MAILMARK,
+    BARCODE_MAILMARK_2D,
+    BARCODE_UPU_S10,
+    BARCODE_MAILMARK_4S,
     BARCODE_AZRUNE,
     BARCODE_CODE32,
     BARCODE_EANX_CC,
@@ -480,8 +497,9 @@ static const char help_message[] = "zint tcl(stub,obj) dll\n"
     "   -bg color: set background color as 6 or 8 hex rrggbbaa\n"
     /* cli option --binary internally handled */
     "   -bind bool: bars above/below the code, size set by -border\n"
+    "   -bindtop bool: bar above the code, size set by -border\n"
     "   -bold bool: use bold text\n"
-    "   -border integer: width of a border around the symbol. Use with -bind/-box 1\n"
+    "   -border integer: width of a border around the symbol. Use with -bind/-box/-bindtop 1\n"
     "   -box bool: box around bar code, size set be -border\n"
     /* cli option --cmyk not supported as no corresponding output */
     "   -cols integer: Codablock F, DotCode, PDF417: number of columns\n"
@@ -521,6 +539,7 @@ static const char help_message[] = "zint tcl(stub,obj) dll\n"
     "   -rotate angle: Image rotation by 0,90 or 270 degrees\n"
     "   -rows integer: Codablock F, PDF417: number of rows\n"
     "   -scale double: Scale the image to this factor\n"
+    "   -scalexdimdp {xdim ?resolution?}: Scale with X-dimension mm, resolution dpmm\n"
     "   -scmvv integer: Prefix SCM with [)>\\R01\\Gvv (vv is integer) (MaxiCode)\n"
     "   -secure integer: EC Level (Aztec, GridMatrix, HanXin, PDF417, QR, UltraCode)\n"
     "   -segN {eci data}: Set the ECI & data content for segment N where N is 1 to 9\n"
@@ -733,6 +752,8 @@ static int Encode(Tcl_Interp *interp, int objc,
     Tcl_Obj *pSegDataObjs[10] = {0};
     Tcl_DString segInputs[10];
     struct zint_seg segs[10];
+    double xdim = 0.0;
+    double resolution = 0.0;
     /*------------------------------------------------------------------------*/
     /* >> Check if at least data and object is given and a pair number of */
     /* >> options */
@@ -758,25 +779,25 @@ static int Encode(Tcl_Interp *interp, int objc,
         /*--------------------------------------------------------------------*/
         /* Option list and indexes */
         static const char *optionList[] = {
-            "-addongap", "-barcode", "-bg", "-bind", "-bold", "-border", "-box",
+            "-addongap", "-barcode", "-bg", "-bind", "-bindtop", "-bold", "-border", "-box",
             "-cols", "-compliantheight", "-dmre", "-dotsize", "-dotty",
             "-eci", "-fast", "-fg", "-format", "-fullmultibyte",
             "-gs1nocheck", "-gs1parens", "-gssep", "-guarddescent",
             "-height", "-heightperrow", "-init", "-mask", "-mode",
             "-nobackground", "-noquietzones", "-notext", "-primary", "-quietzones",
-            "-reverse", "-rotate", "-rows", "-scale", "-scmvv", "-secure",
+            "-reverse", "-rotate", "-rows", "-scale", "-scalexdimdp", "-scmvv", "-secure",
             "-seg1", "-seg2", "-seg3", "-seg4", "-seg5", "-seg6", "-seg7", "-seg8", "-seg9",
             "-separator", "-smalltext", "-square", "-structapp",
             "-to", "-vers", "-vwhitesp", "-werror", "-whitesp",
             NULL};
         enum iOption {
-            iAddonGap, iBarcode, iBG, iBind, iBold, iBorder, iBox,
+            iAddonGap, iBarcode, iBG, iBind, iBindTop, iBold, iBorder, iBox,
             iCols, iCompliantHeight, iDMRE, iDotSize, iDotty,
             iECI, iFast, iFG, iFormat, iFullMultiByte,
             iGS1NoCheck, iGS1Parens, iGSSep, iGuardDescent,
             iHeight, iHeightPerRow, iInit, iMask, iMode,
             iNoBackground, iNoQuietZones, iNoText, iPrimary, iQuietZones,
-            iReverse, iRotate, iRows, iScale, iSCMvv, iSecure,
+            iReverse, iRotate, iRows, iScale, iScaleXdimDp, iSCMvv, iSecure,
             iSeg1, iSeg2, iSeg3, iSeg4, iSeg5, iSeg6, iSeg7, iSeg8, iSeg9,
             iSeparator, iSmallText, iSquare, iStructApp,
             iTo, iVers, iVWhiteSp, iWError, iWhiteSp
@@ -797,6 +818,7 @@ static int Encode(Tcl_Interp *interp, int objc,
         /* >> Decode object */
         switch (optionIndex) {
         case iBind:
+        case iBindTop:
         case iBold:
         case iBox:
         case iCompliantHeight:
@@ -931,6 +953,13 @@ static int Encode(Tcl_Interp *interp, int objc,
                 my_symbol->output_options &= ~BARCODE_BIND;
             }
             break;
+        case iBindTop:
+            if (intValue) {
+                my_symbol->output_options |= BARCODE_BIND_TOP;
+            } else {
+                my_symbol->output_options &= ~BARCODE_BIND_TOP;
+            }
+            break;
         case iBold:
             if (intValue) {
                 my_symbol->output_options |= BOLD_TEXT;
@@ -1044,7 +1073,6 @@ static int Encode(Tcl_Interp *interp, int objc,
         case iFG:
             strncpy(my_symbol->fgcolour, pStr, lStr);
             my_symbol->fgcolour[lStr]='\0';
-            printf("my_symbol->fgcolour %s\n", my_symbol->fgcolour);
             break;
         case iBG:
             strncpy(my_symbol->bgcolour, pStr, lStr);
@@ -1091,6 +1119,42 @@ static int Encode(Tcl_Interp *interp, int objc,
                 my_symbol->scale = (float)doubleValue;
             }
             break;
+        case iScaleXdimDp:
+            /* >> Decode the -scalexdimdp parameter as list of xdim ?resolution? */
+            {
+                Tcl_Obj *poParam;
+                xdim = resolution = 0.0;
+                if (TCL_OK != Tcl_ListObjLength(interp,
+                    objv[optionPos+1], &lStr))
+                {
+                    fError = 1;
+                } else if ( ! ( lStr == 1 || lStr == 2 ) ) {
+                    Tcl_SetObjResult(interp,
+                        Tcl_NewStringObj(
+                        "option -scalexdimdp not a list of 1 or 2", -1));
+                    fError = 1;
+                } else {
+                    if (TCL_OK != Tcl_ListObjIndex(interp, objv[optionPos+1],
+                        0, &poParam)
+                        || TCL_OK != Tcl_GetDoubleFromObj(interp, poParam, &xdim)
+                        || xdim < 0.0)
+                    {
+                        fError = 1;
+                    }
+                    if (!fError && lStr == 2 && (
+                        TCL_OK != Tcl_ListObjIndex(interp, objv[optionPos+1],
+                            1, &poParam)
+                        || TCL_OK != Tcl_GetDoubleFromObj(interp, poParam, &resolution)
+                        || resolution < 0.0))
+                    {
+                        fError = 1;
+                    }
+                    if (!fError && resolution == 0.0) {
+                        resolution = 12.0; /* Default 12 dpmm (~300 dpi) */
+                    }
+                }
+            }
+            break;
         case iBorder:
             if (intValue < 0 || intValue > 1000) {
                 Tcl_SetObjResult(interp,
@@ -1366,6 +1430,16 @@ static int Encode(Tcl_Interp *interp, int objc,
             my_symbol->option_1 = rows;
         }
     }
+    if (resolution) {
+        float scale;
+        if (xdim == 0.0) {
+            xdim = ZBarcode_Default_Xdim(my_symbol->symbology);
+        }
+        scale = ZBarcode_Scale_From_XdimDp(my_symbol->symbology, (float)xdim, (float)resolution, NULL /*filetype*/);
+        if (scale > 0.0f) {
+            my_symbol->scale = scale;
+        }
+    }
     /*------------------------------------------------------------------------*/
     /* >>> Prepare input dstring and encode it to ECI encoding*/
     Tcl_DStringInit(& dsInput);
